@@ -1,16 +1,22 @@
-# Copyright (c) 2009-2012 Martin Becker.  All rights reserved.
+# Copyright (c) 2009-2015 Martin Becker.  All rights reserved.
 # This package is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 #
-# $Id: ModInt.pm 36 2012-08-26 22:25:41Z demetri $
+# $Id: ModInt.pm 51 2015-04-29 22:09:44Z demetri $
 
 package Math::ModInt;
 
 use 5.006;
 use strict;
+use warnings;
 use Math::ModInt::Event
     qw(UsageError Nonexistent LoadingFailure UndefinedResult DifferentModuli);
 
+sub _no_op {
+    my ($op) = @_;
+    return $op => sub { UsageError->raise('undefined operation', $op) };
+}
+
 use overload (
     'neg'      => '_NEG',
     '+'        => \&_oadd,
@@ -25,6 +31,8 @@ use overload (
     '0+'       => 'residue',
     '""'       => 'as_string',
     'fallback' => undef,
+    _no_op('abs'),
+    _no_op('int'),
 );
 
 # ----- class data -----
@@ -34,7 +42,7 @@ BEGIN {
     our @ISA       = qw(Exporter);
     our @EXPORT_OK = qw(mod);
     our @CARP_NOT  = qw(Math::ModInt::ChineseRemainder);
-    our $VERSION   = 0.007;
+    our $VERSION   = '0.008';
 }
 
 sub _max_modulus_perl {
@@ -52,9 +60,6 @@ use constant _MAX_MODULUS_PERL => _max_modulus_perl();
 my $undefined = bless [];                       # singleton
 my %loaded = ();                                # collects loaded modules
 
-my $check_pari  = 1;
-my $use_pari;
-
 # ----- private subroutines -----
 
 sub _is_integer {
@@ -64,8 +69,7 @@ sub _is_integer {
         eval {
             use warnings FATAL => 'all';
             ref $arg?
-                $arg->isa('Math::BigInt') ||
-                overload::Method($arg, '|') && $arg == ($arg | 0)
+                $arg->isa('Math::BigInt')
             :
                 $arg == int($arg) && abs($arg) <= ~0
         };
@@ -208,6 +212,14 @@ sub signed_residue {
     return $n <= $r? -$n: $r;
 }
 
+sub centered_residue {
+    my ($this) = @_;
+    my $r = $this->residue;
+    my $m = $this->modulus;
+    my $n = $m - $r;
+    return $n < $r? -$n: $r;
+}
+
 sub is_defined {
     my ($this) = @_;
     return ref $undefined ne ref $this;
@@ -264,7 +276,7 @@ Math::ModInt - modular integer arithmetic
 
 =head1 VERSION
 
-This documentation refers to version 0.007 of Math::ModInt.
+This documentation refers to version 0.008 of Math::ModInt.
 
 =head1 SYNOPSIS
 
@@ -396,6 +408,12 @@ as possible.  In case of a tie, i.e. when the modulus is an even
 number and the residue is half the modulus, the negative value is
 given preference (like in many native signed integer formats).
 
+=item I<centered_residue>
+
+The object method C<$x-E<gt>centered_residue> is equivalent to
+C<$x-E<gt>signed_residue> except that the positive value is given
+preference when the residue is precisely half the modulus.
+
 =item I<is_zero>
 
 =item I<is_not_zero>
@@ -421,7 +439,7 @@ in case of the C<undefined> placeholder.
 
 =item I<optimize_time>
 
-Some implementations can employ different optimization strategys
+Some implementations can employ different optimization strategies
 for either time or space efficiency.  Time efficiency aims to speed
 up repetitive calculations at the expense of memory space.  Space
 efficiency aims to minimize the memory footprint at the expense of
@@ -557,10 +575,11 @@ division only makes sense if the denominator residue is coprime
 to the modulus.  Operands with different moduli generally can not
 be combined in binary operations.
 
-By default, operations with incompatible operands consistently yield
-the Math::ModInt->undefined object, which will raise an exception
-upon modulus/residue inspection, but can be recogized by the boolean
-result of the is_defined/is_undefined methods.
+By default, operations with incompatible operands or undefined
+results consistently yield the Math::ModInt->undefined object, which
+will raise an exception upon modulus/residue inspection, but can
+be recognized by the boolean result of the is_defined/is_undefined
+methods.  See L<Math::ModInt::Event> for ways to alter this behaviour.
 
 =head1 DEPENDENCIES
 
@@ -649,7 +668,7 @@ hall of fame.
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2009-2012 by Martin Becker.  All rights reserved.
+Copyright (c) 2009-2015 by Martin Becker.  All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.6.0 or,
